library ieee;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.ALL;
use ieee.std_logic_unsigned.ALL;
use ieee.numeric_std.ALL;
use ieee.numeric_bit.ALL;

entity drp_fsm is
   port ( 
	  Mul_Div         : in 	 std_logic_vector(15 downto 0);
	  Rst_in          : in	 std_logic;
	  go              : in	 std_logic;
	  CLKIN_IN        : in    std_logic; 
	  Rst_out			  : out   std_logic;
          DADDR_IN        : out   std_logic_vector (6 downto 0);
          DEN_IN          : out   std_logic; 
          DI_IN           : out   std_logic_vector (15 downto 0); 
          DWE_IN          : out   std_logic; 
          DO_OUT          : in    std_logic_vector (15 downto 0); 
          DRDY_OUT        : in    std_logic;
          LOCKED_OUT      : in    std_logic
);
end drp_fsm;

architecture FSM of drp_fsm is

component multiplier IS
	port (
	a: in std_logic_vector(7 downto 0);
	p: out std_logic_vector(11 downto 0));
END component;

component div_gen_v3_0 is
  port (
    rfd : out STD_LOGIC; 
    clk : in STD_LOGIC := 'X'; 
    dividend : in STD_LOGIC_VECTOR ( 11 downto 0 ); 
    quotient : out STD_LOGIC_VECTOR ( 11 downto 0 ); 
    divisor : in STD_LOGIC_VECTOR ( 7 downto 0 ); 
    fractional : out STD_LOGIC_VECTOR ( 1 downto 0 ) 
  );
end component;

	type state_type is (idle, CLEAR_DCM, CLEAR_WAIT, READ_MODE, RMOD_WAIT, CHANGE_MODE,
	CMOD_WAIT, CLEAR_DCM_2, CLEAR_DCM_3, WAIT_LOCKED, LOCKED_DONE, CLEAR_DCM_W, CLEAR_WAIT_W,
	READ_DCM, READ_WAIT, WRITE_DCM, WRITE_WAIT, WRITE_FINISH, WRITE_FINISH_2, WRITE_FINISH_3);
	signal next_state, current_state: state_type;
	signal tmp : std_logic_vector(15 downto 0);
	signal data_addr : std_logic_vector(6 downto 0);
	signal act : std_logic;
	signal data_en : std_logic;
	signal data_write : std_logic;
	signal data_in : std_logic_vector(15 downto 0);
	signal data_out : std_logic_vector(15 downto 0);
	signal tmp_integer : integer;
	signal Rst_tmp : std_logic;
	signal cnt: std_logic_vector(0 to 31);
        signal dcm_ready : std_logic:='1';
	signal high_freq_mode : std_logic;
	signal mul_10: std_logic_vector(11 downto 0);
	signal result : std_logic_vector(11 downto 0);
	signal rfd : std_logic;
	signal fractional : STD_LOGIC_VECTOR ( 1 downto 0);
	signal mul_1 : std_logic_vector(7 downto 0);
	signal div_1 : std_logic_vector(7 downto 0);
begin

	DEN_IN <= data_en;
	DWE_IN <= data_write;
	DADDR_IN <= data_addr;
	DI_IN <= data_in;
	Rst_out <= Rst_tmp or Rst_in;

	mul_1 <= Mul_Div(15 downto 8) + '1';
    mult : multiplier
  port map(
  a => mul_1,
  p => mul_10
  );

div_1 <= Mul_Div(7 downto 0) +'1';

    div : div_gen_v3_0
  port map(
    rfd => rfd,
    clk => CLKIN_IN,
    dividend => mul_10,
    quotient => result,
    divisor => div_1,
    fractional => open
  );

  check_mode: process(result)
  begin
  if (conv_integer(result) > 14) then high_freq_mode <= '1';
  else high_freq_mode <= '0';
  end if;
  end process;

	action: process(go)
	begin
		if ((go ='1') and (dcm_ready = '1')) then 
			act <= '1';
		else act <= '0';
		end if;
	end process;

	state_reg: process(CLKIN_IN, Rst_in)
	begin
		if Rst_in = '1' then
			current_state <= idle;
		elsif (rising_edge(CLKIN_IN)) then 
			current_state <= next_state;
		end if;
	end process;
 
	FSM_DRP : process(current_state, go, DRDY_OUT)

	begin
		case current_state is
			when idle =>		data_en <= '0';
									dcm_ready <= '1';
									data_write <= '0';
									Rst_tmp <= '0';
									data_out <= (others => '0');
									if (act ='1') then
										next_state <= CLEAR_DCM;
									else
										next_state <= idle;
									end if;

			when CLEAR_DCM =>
									dcm_ready <= '0';
									data_en <= '1';
									data_write <= '0';
									data_addr <= "0000000";--00h
									Rst_tmp <= '0';
									data_out <= DO_OUT;
									next_state <= CLEAR_WAIT;

			when CLEAR_WAIT =>
									dcm_ready <= '0';
									data_en <= '0';
									data_write <= '0';
									data_addr <= "0000000";--00h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									if (DRDY_OUT ='0') then
									next_state <= CLEAR_WAIT;
									else next_state <= READ_MODE;
									end if;
				
			when READ_MODE => dcm_ready <= '0';
									data_en <= '1';
									data_write <= '0';
									data_addr <= "1000001";--41h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									next_state <= RMOD_WAIT;

			when RMOD_WAIT => dcm_ready <= '0';
									data_en <= '0';
									data_write <= '0';
									data_addr <= "1000001";--41h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									if (DRDY_OUT ='0') then
									next_state <= RMOD_WAIT;
									else next_state <= CHANGE_MODE;
									end if;
			when CHANGE_MODE=>
									dcm_ready <= '0';
									data_en <= '1';
									data_write <= '1';
									data_addr <= "1000001";--41h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									data_in(7 downto 0)<= ("00000" & high_freq_mode & "00");
									next_state <= CMOD_WAIT;

			when CMOD_WAIT =>
									dcm_ready <= '0';
									data_en <= '0';
									data_write <= '0';
									data_addr <= "1000001";--41h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									if (DRDY_OUT ='0') then
									next_state <= CMOD_WAIT;
									else next_state <= CLEAR_DCM_2;
									end if;

			when CLEAR_DCM_2 =>
									dcm_ready <= '0';
									data_en <= '1';
									data_write <= '0';
									data_addr <= "0000000";--00h
									Rst_tmp <= '0';
									data_out <= DO_OUT;
									next_state <= CLEAR_DCM_3;

			when CLEAR_DCM_3 =>
									dcm_ready <= '0';
									data_en <= '0';
									data_write <= '0';
									data_addr <= "0000000";--00h
									Rst_tmp <= '0';
									data_out <= DO_OUT;
									next_state <= WAIT_LOCKED;
			when WAIT_LOCKED =>		
									dcm_ready <= '0';
									if (LOCKED_OUT = '1') then
									next_state <= LOCKED_DONE;
									-- dcm_ready <= '1';
									else next_state <= WAIT_LOCKED;
									-- dcm_ready <= '0';
									end if;

			when LOCKED_DONE =>		
									dcm_ready <= '0';
									next_state <= CLEAR_DCM_W;

			when CLEAR_DCM_W =>
									dcm_ready <= '0';
									data_en <= '1';
									data_write <= '0';
									data_addr <= "0000000";--00h
									Rst_tmp <= '0';
									data_out <= DO_OUT;
									next_state <= CLEAR_WAIT_W;

			when CLEAR_WAIT_W =>
									dcm_ready <= '0';
									data_en <= '0';
									data_write <= '0';
									data_addr <= "0000000";--00h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									if (DRDY_OUT ='0') then
									next_state <= CLEAR_WAIT_W;
									else next_state <= READ_DCM;
									end if;
									
			when READ_DCM =>	
									dcm_ready <= '0';
									data_en <= '1';
									data_write <= '0';
									data_addr <= "1010000";--50h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									data_in <= Mul_Div;
									next_state <= READ_WAIT;								
			when READ_WAIT =>
									dcm_ready <= '0';
									data_en <= '0';
									data_write <= '0';
									data_addr <= "1010000";--50h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									data_in <= Mul_Div;
									if (DRDY_OUT ='0') then
									next_state <= READ_WAIT;
									else next_state <= WRITE_DCM;
									end if;
									
			when WRITE_DCM =>	
									dcm_ready <= '0';
									data_en <= '1';
									data_write <= '1';
									data_addr <= "1010000";--50h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									data_in <= Mul_Div;
									next_state <= WRITE_WAIT;

			when WRITE_WAIT =>
									dcm_ready <= '0';
									data_en <= '0';
									data_write <= '0';
									data_addr <= "1010000";--50h
									Rst_tmp <= '1';
									data_out <= DO_OUT;
									data_in <= Mul_Div;
									if (DRDY_OUT ='0') then
									next_state <= WRITE_WAIT;
									else next_state <= WRITE_FINISH;
									end if;

			when WRITE_FINISH =>		
									dcm_ready <= '0';
									data_en <= '1';
									data_write <= '0';
									Rst_tmp <= '0';
									data_addr <= "0000000";--00h
									data_out <= DO_OUT;
									next_state <= WRITE_FINISH_2;

			when WRITE_FINISH_2 =>		
									dcm_ready <= '0';
									data_en <= '0';
									data_write <= '0';
									Rst_tmp <= '0';
									data_addr <= "0000000";--00h
									data_out <= DO_OUT;
									if (LOCKED_OUT ='0') then
									next_state <= WRITE_FINISH_2;
									else next_state <= WRITE_FINISH_3;
									dcm_ready <= '1';
									end if;

			when WRITE_FINISH_3 =>	dcm_ready <= '1';
									next_state <= idle;
		end case;
	end process;
end architecture FSM;
