/**********************************************************************
 * University of Rennes 1 - IRISA.
 *
 * FileName:        bit2uparc.c
 * Dependencies:    Header (.h) files if applicable, see below
 * Processor:       
 * Compiler:        gcc
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Author        Date      		Comments on this revision
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Bonamy R.		03/11/2012		version 1
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * ADDITIONAL NOTES:
 * Code Tested on: Virtex 5 and Virtex 6 platform
 *
 * DESCRIPTION:
 *Add required information at the beginning of a partial bitstream to be hendled by UPaRC
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define SIZE 2200000

/* Used for parsing bitstream header */
#define XHI_EVEN_MAGIC_BYTE     0x0f
#define XHI_ODD_MAGIC_BYTE      0xf0
#define XHI_BIT_HEADER_FAILURE -1

/// This typedef contains bitstream header information.
typedef struct
{
    unsigned int HeaderLength;     /* Length of header in 32 bit words */
    unsigned int BitstreamLength;  /* Length of bitstream to read in bytes*/
    unsigned char DesignName[40];       /* Design name read from bitstream header */
    unsigned char PartName[40];         /* Part name read from bitstream header */
    unsigned char Date[40];             /* Date read from bitstream header */
    unsigned char Time[40];             /* Bitstream creation time read from header */
    unsigned int MagicLength;      /* Length of the magic numbers in header */

} XHwIcap_Bit_Header;



XHwIcap_Bit_Header XHwIcap_ReadHeader(unsigned char *Data);

main(int argc, char **argv)
{
	if (argc<3)
	{
	  printf("This program converts bitstream -.bit file- to bitstream to load directly to UPaRC.\n");
	  printf("Bitstream input file and destination file required\n");
	  return 1;
	}
	
	XHwIcap_Bit_Header bitheader;
	FILE *file, *file2;
	unsigned char *buffer, *pos;
	unsigned long fileLen;
	int i;

	pos=strrchr(argv[1], '.');
	//printf("ext: %s\n", pos);
	if (strcmp(pos, ".bit"))
	{
	  	fprintf(stderr, "%s - Please provide .bit file!!!\n", argv[1]);
		return 1;
	}
	
	file = fopen(argv[1], "rb");
	if (!file)
	{
		fprintf(stderr, "Can't open file %s\n", argv[1]);
		return 1;
	}
	
	file2 = fopen(argv[2], "w");
	if (!file)
	{
		fprintf(stderr, "Can't create file %s\n", argv[2]);
		return 1;
	}

	///gets file length to allocate buffer size
	fseek(file, 0, SEEK_END);
	fileLen=ftell(file);
	fseek(file, 0, SEEK_SET);

	buffer=(unsigned char *)malloc(fileLen+1);
	if (!buffer)
	{
		fprintf(stderr, "Can't allocate memory for buffer size: %ld\nMemory error!", fileLen);
		fclose(file);
		return 1;
	}

	///load bitstream file
	fread(buffer, fileLen, 1, file);
	fclose(file);
	
	bitheader=XHwIcap_ReadHeader(buffer);
	printf("Design   Name : %s\n",bitheader.DesignName);
	printf("Part     Name : %s\n",bitheader.PartName);
	printf("Creation Time : %s - %s\n",bitheader.Date, bitheader.Time);
	printf("HeaderLength  : %d\n",bitheader.HeaderLength);
	printf("BitstreamLength  : %d\n",bitheader.BitstreamLength);

	
	bit2uparc(buffer, fileLen, bitheader, argv[1], file2);
	
	free(buffer);
	fclose(file2);
	printf("DONE\n");
}


bit2uparc(unsigned char *buffer, int buffer_size, XHwIcap_Bit_Header bitheader, char* filename, FILE *file)
{
  int i;
  unsigned long word;
  struct tm *local;
  time_t t;
  
  t = time(NULL);
  local = localtime(&t);
  
    
  ///generate bitstream length at the beginning of memory (for UPaRC standalone)
  
  fprintf(file,"%c", (bitheader.BitstreamLength)>>24);
  fprintf(file,"%c", ((bitheader.BitstreamLength)>>16) & 0xFF);
  fprintf(file,"%c", ((bitheader.BitstreamLength)>>8) & 0xFF);
  fprintf(file,"%c", ((bitheader.BitstreamLength)) & 0xFF);
  
  
  ///generate whole bitstream data to bitstream file
  for(i = bitheader.HeaderLength;i < bitheader.BitstreamLength;i+=4)
  {
    ///print binary data to destination file up to bitstream end
    fprintf(file,"%c",buffer[i]);
    fprintf(file,"%c",buffer[i+1]);
    fprintf(file,"%c",buffer[i+2]);
    fprintf(file,"%c",buffer[i+3]);
  }
}

/* Parse Bitfile header */
XHwIcap_Bit_Header XHwIcap_ReadHeader(unsigned char *Data) 
{
    unsigned int I;   
    unsigned int Len;  
    unsigned int Tmp;  
    XHwIcap_Bit_Header Header;
    unsigned int Index;

    /* Start Index at start of bitstream */
    Index=0;

    /* Initialize HeaderLength.  If header returned early inidicates
     * failure. 
     */
    Header.HeaderLength = XHI_BIT_HEADER_FAILURE;

    /* Get "Magic" length */
    Header.MagicLength = Data[Index++];
    Header.MagicLength = (Header.MagicLength << 8) | Data[Index++];

    /* Read in "magic" */
    for (I=0; I<Header.MagicLength-1; I++) {
        Tmp = Data[Index++];
        if (I%2==0 && Tmp != XHI_EVEN_MAGIC_BYTE) 
        {
          return Header;   /* INVALID_FILE_HEADER_ERROR */
        }
        if (I%2==1 && Tmp != XHI_ODD_MAGIC_BYTE) 
        {
            return Header;   /* INVALID_FILE_HEADER_ERROR */
        }
    }

    /* Read null end of magic data. */
    Tmp = Data[Index++];

    /* Read 0x01 (short) */
    Tmp = Data[Index++];
    Tmp = (Tmp << 8) | Data[Index++];

    /* Check the "0x01" half word */
    if (Tmp != 0x01) 
    {
       return Header;   /* INVALID_FILE_HEADER_ERROR */
    }


    /* Read 'a' */
    Tmp = Data[Index++];
    if (Tmp != 'a') 
    {
        return Header;    /* INVALID_FILE_HEADER_ERROR  */
    }

    /* Get Design Name length */
    Len = Data[Index++];
    Len = (Len << 8) | Data[Index++];

    /* allocate space for design name and final null character. */
    //Header.DesignName = (Xuint8 *)malloc(Len);

    /* Read in Design Name */
    for (I=0; I<Len; I++) 
    {
        Header.DesignName[I] = Data[Index++];
    }

    /* Read 'b' */
    Tmp = Data[Index++];
    if (Tmp != 'b') 
    {
        return Header;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get Part Name length */
    Len = Data[Index++];
    Len = (Len << 8) | Data[Index++];

    /* allocate space for part name and final null character. */
    //Header.PartName = (Xuint8 *)malloc(Len);

    /* Read in part name */
    for (I=0; I<Len; I++) 
    {
        Header.PartName[I] = Data[Index++];
    }

    /* Read 'c' */
    Tmp = Data[Index++];
    if (Tmp != 'c') 
    {
        return Header;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get date length */
    Len = Data[Index++];
    Len = (Len << 8) | Data[Index++];

    /* allocate space for date and final null character. */
    //Header.Date = (Xuint8 *)malloc(Len);

    /* Read in date name */
    for (I=0; I<Len; I++) 
    {
        Header.Date[I] = Data[Index++];
    }

    /* Read 'd' */
    Tmp = Data[Index++];
    if (Tmp != 'd') 
    {
        return Header;  /* INVALID_FILE_HEADER_ERROR  */
    }

    /* Get time length */
    Len = Data[Index++];
    Len = (Len << 8) | Data[Index++];

    /* allocate space for time and final null character. */
    //Header.Time = (Xuint8 *)malloc(Len);

    /* Read in time name */
    for (I=0; I<Len; I++) 
    {
        Header.Time[I] = Data[Index++];
    }

    /* Read 'e' */
    Tmp = Data[Index++];
    if (Tmp != 'e') 
    {
        return Header;  /* INVALID_FILE_HEADER_ERROR */
    }

    /* Get byte length of bitstream */
    Header.BitstreamLength = Data[Index++];
    Header.BitstreamLength = (Header.BitstreamLength << 8) | Data[Index++];
    Header.BitstreamLength = (Header.BitstreamLength << 8) | Data[Index++];
    Header.BitstreamLength = (Header.BitstreamLength << 8) | Data[Index++];

    Header.HeaderLength = Index;

    return Header;

}

